// AuxSen Analysis macro written by Andre C. Stiel, 2020 for Herud-Sikimic et al, Nature, 2021 


// USER SETTINGS ///////////////////////////////////////////////////////////////////////
GUI = true;	// Set to "true" if GUI should be used, otherwise script runs with setting in the below section

// working folder. All .czi files in this folder are processed. A subfolder "" is generated which stores all results
folder = "/Users/andre.stiel/Documents/writing/articles/2016_auxin_big/Martina/testproblem/";

// ROI slection
ROI_automatic = "automatic";	// 0=manual, 1=automatic, if ROI selection is automatic, this only works in very homogenous data and with the right numbers (Thr_from and Thr_then) determined in manual mode before
Thr_from = 25;		// see above
Thr_then = -1;		// see above
S = 15;			// thresholding of the ROI segmentation, default=15 in micron^2

// tip selection
tipauto = "line";		// tipfinder 0=manual (click position with left mouse on open image), 1= autotmatic (set "tipfind_thresh" for sensitivity)
tipfind_thresh = 10000; // The lower the value the more sensitive the tip detection. Might then also pick up noise as tip.
BG = 100;		// Everything below considered as out of root BG

// FRET analysis
FRET_analysis = "unmixing";	// 1 = simnple division, 2 = based on unmixing results
FRET_thresh_min = 0.3;		// lower cut of for FRET ratio signals
FRET_thresh_max = 5;		// upper cut of for FRET ratio signals

// Visulaization
Visu_style = "fixed" 		// 1  = dynamic thresholds based on "offset_div" below, 2 = fixed threshold based on "upper" and "lower" below. 
offset_div = 1.5;   	// x times the std deviation +/- from mean. E.g. for 1:  lower bound: "mean - std * 1", upper bound : "mean + std * 1"
lower = 50;		// lower limit of the colorrange	
upper = 400;		// upper limit of the colorrange	
area_thresh=10;  	// only consider ROIs above this pixel number for analysis

// Script
deb = false; 		// if deb = 1 a number of intermediate images are saved
inv = true;		// toggles invert on / off, might be necessary on a number of systems, 1 = default

// path to matrix used for multi-channel unmixing ("unfret"). This path has to point to a prior generated unmixing-matrix .txt generated with Spectral Unmix” version 1.3 (by Joachim Walter, https://imagej.nih.gov/ij/plugins/spectral-unmixing.html) 
matrix_path = "/Users/andre.stiel/Documents/writing/articles/2016_auxin_big/Martina/2019-08-20_Unmixuing_matrix_AuxSen_channal_1-5.txt";  		
preset_path = "/Users/andre.stiel/.IJ_AuxSenMacro.txt";  // here the last used settings are stored when using the GUI and reloaded at restart- this does not alter the setting above


/////// G U I ////////////////////////////////////////////////////
/// Section needed for GUI appearance ////
if (GUI == true){


	if (File.exists(preset_path) == true){
		print("LOAD preset");
		preset_string = File.openAsString(preset_path);
		lines=split(preset_string,"\n");
		folder = lines[1];
	        ROI_automatic = lines[2];
	        Thr_from = parseFloat(lines[3]);
	        Thr_then = parseFloat(lines[4]);
	        S = parseFloat(lines[5]);
	        tipauto = lines[6];
	        tipfind_thresh = parseFloat(lines[7]);
	        BG = parseFloat(lines[8]);
	        FRET_analysis = lines[9];
	        FRET_thresh_min = parseFloat(lines[10]);
	        FRET_thresh_max = parseFloat(lines[11]);
	        Visu_style = lines[12];
	        offset_div = parseFloat(lines[13]);
	        area_thresh = parseFloat(lines[14]);
	        lower = parseFloat(lines[15]);
	        upper = parseFloat(lines[16]);
	        deb = lines[17];
	        inv = lines[18];
	        matrix_path = lines[19];
		}
	else{
		print("no preset");
		}
	
	



	// Dialog window for parameters choice
	macro_path = getInfo("macro.filepath");
	Dialog.create("AuxSen - FRET Analysis Macro v 1.0");
	Dialog.addMessage("FRET Analysis Macro for AuxSen - a transgene Indole-3-acetic acid (IAA) sensor, v. 1.0");
	Dialog.addMessage("Published in: Herud, ????, ???\nPlease direct questions regarding the macro at andre.stiel@helmholtz-muenchen.de");
	//Dialog.addDirectory("Choose directory to process", macro_path);
	Dialog.addString("Path to folder with images to read (with trailing slash or backslash)", folder, 120);
	if (ROI_automatic == "manual"){
		items = newArray("manual", "automatic");
		}
	else{
		items = newArray("automatic", "manual");
		}
	Dialog.addChoice("Select Mode of ROI detection", items) 
	Dialog.addNumber("threshold from (only for 'automatic')", Thr_from);
	Dialog.addNumber("threshold then (only for 'automatic')", Thr_then);
	Dialog.addNumber("minimal ROI size in micron^2", S);
	if (tipauto == "manual"){
		items = newArray("manual", "automatic", "line");
		} 
	else if(tipauto == "automatic"){
		items = newArray("automatic", "manual", "line");
		}	 
	else{
		items = newArray("line", "automatic", "manual");
		}
	Dialog.addChoice("Select Mode of tip detection", items);
	Dialog.addNumber("Tip finder threshold", tipfind_thresh);
	Dialog.addNumber("Center of mass finder, BG threshold", BG);
	if (FRET_analysis == "division"){
		items = newArray("division", "unmixing");
		} 
	else{
		items = newArray("unmixing", "division");
		}	 
	Dialog.addChoice("Select Mode of FRET analysis", items);
	Dialog.addNumber("Min. FRET threshold", FRET_thresh_min);
	Dialog.addNumber("Max. FRET threshold", FRET_thresh_max);
	if (Visu_style == "dynamic"){
		items = newArray("dynamic", "fixed");
                } 
        else{   
		items = newArray("fixed", "dynamic");
                } 
	Dialog.addChoice("Visualization dynamic range", items);
	Dialog.addNumber("Times std. (only for 'dynamic')", offset_div);
	Dialog.addNumber("Only color ROIs with an area > (only for 'dynamic')", area_thresh);
	Dialog.addNumber("Colorrange minimum (only for 'fixed')", lower);
	Dialog.addNumber("Colorrange maximum (only for 'fixed')", upper);
	Dialog.addMessage("\n Script setting ");
	Dialog.addCheckbox("Debug mode", deb);
	Dialog.addCheckbox("Invert (may be necessary on some OS", inv);
	Dialog.addString("Path to unmixing matrix (only for 'unmixing')", matrix_path,120); 
	
	
	
	Dialog.show();
	folder = Dialog.getString();
	ROI_automatic = Dialog.getChoice();
	Thr_from = Dialog.getNumber();
	Thr_then = Dialog.getNumber();
	S = Dialog.getNumber();
	tipauto = Dialog.getChoice();
	tipfind_thresh = Dialog.getNumber();
	BG = Dialog.getNumber();
	FRET_analysis = Dialog.getChoice();
	FRET_thresh_min = Dialog.getNumber();
	FRET_thresh_max = Dialog.getNumber();
	Visu_style = Dialog.getChoice();
	offset_div = Dialog.getNumber();
	area_thresh = Dialog.getNumber();
	lower = Dialog.getNumber();
	upper = Dialog.getNumber();
	deb = Dialog.getCheckbox();
	inv = Dialog.getCheckbox();
	matrix_path = Dialog.getString();


	//write to preset file
	print("WRITE preset");
	File.saveString("ImageJ macro AuxSen setting save file\n",preset_path);
        File.append(folder,preset_path);
        File.append(ROI_automatic,preset_path);
        File.append(d2s(Thr_from,2),preset_path);
        File.append(d2s(Thr_then,2),preset_path);
        File.append(d2s(S,2),preset_path);
        File.append(tipauto,preset_path);
        File.append(d2s(tipfind_thresh,2),preset_path);
        File.append(d2s(BG,2),preset_path);
        File.append(FRET_analysis,preset_path);
        File.append(d2s(FRET_thresh_min,2),preset_path);
        File.append(d2s(FRET_thresh_max,2),preset_path);
        File.append(Visu_style,preset_path);
        File.append(d2s(offset_div,2),preset_path);
        File.append(d2s(area_thresh,2),preset_path);
        File.append(d2s(lower,2),preset_path);
        File.append(d2s(upper,2),preset_path);
        File.append(deb,preset_path);
        File.append(inv,preset_path);
        File.append(matrix_path,preset_path);
	}










// BODY /////////////////////////////////////////////////////////////////////////////////
cs = "Check String\n\n";
dir_list = getFileList(folder);
cs = cs+"Path to read image files from"+folder+"\n\n";
// !!!!!!!!!!!!!! ADD POS FILE GENERATE
getDateAndTime(year, month, dayOfWeek, dayOfMonth, hour, minute, second, msec);
month = month + 1;
save_folder = d2s(year,0)+"_"+d2s(month,0)+"_"+d2s(dayOfMonth,0)+"_"+d2s(hour,0)+"_"+d2s(minute,0)+"_"+d2s(second,0);
cs = cs+"Run script at:"+d2s(year,0)+"_"+d2s(month,0)+"_"+d2s(dayOfMonth,0)+"_"+d2s(hour,0)+"_"+d2s(minute,0)+"_"+d2s(second,0)+"\n\n";
cs = cs+"Save folder:\t"+save_folder+"\n\n";
File.makeDirectory(folder+"/"+save_folder);
File.saveString("\n ",folder+"/"+save_folder+"/pos.txt");
setOption("ExpandableArrays", true);
cs = cs+"--------- SETTINGS --------\n";
cs = cs+"ROI selection:\t"+ROI_automatic+"\n"; 
cs = cs+"ROI selection threshold from:\t"+d2s(Thr_from,2)+"\n"; 
cs = cs+"ROI selection threhold then:\t"+d2s(Thr_then,2)+"\n";
cs = cs+"Minimal ROI size:\t"+d2s(S,2)+"\n"; 
cs = cs+"Tip find mode:\t"+tipauto+"\n"; 
cs = cs+"Tip find threshold:\t"+d2s(tipfind_thresh,2)+"\n"; 
cs = cs+"Tip find BG:\t"+d2s(BG,2)+"\n"; 
cs = cs+"FRET analysis type:\t"+d2s(FRET_analysis,2)+"\n"; 
cs = cs+"FRET analysis threshold:\t"+d2s(FRET_thresh_min,2)+"\n"; 
cs = cs+"FRET analysis threshold:\t"+d2s(FRET_thresh_max,2)+"\n"; 
cs = cs+"Visualization colorscale style:\t"+Visu_style+"\n"; 
cs = cs+"Offset division for dynamic colorscale:\t"+d2s(offset_div,2)+"\n"; 
cs = cs+"Area thresh for dynamic colorscale:\t"+d2s(area_thresh,2)+"\n"; 
cs = cs+"Colorscale lower value for fixed scale:\t"+d2s(lower,2)+"\n"; 
cs = cs+"Colorscale upper value for fixed scale:\t"+d2s(upper,2)+"\n"; 
cs = cs+"Debug mode:\t"+deb+"\n"; 
cs = cs+"Invert:\t"+inv+"\n"; 
cs = cs+"Unmixing Matrix path:\t"+matrix_path+"\n"; 
cs = cs+"\n\n\n";






for (i=0; i<dir_list.length; i++) {
	if (endsWith(dir_list[i], ".czi")) {
		print(dir_list[i]+"----------------------------------");

	// Load file using "Bio-Formats Importer"
		print("Loading Files");
		file = dir_list[i];
		file_path = folder+file;
		cs = cs+"Processed File:\t"+file_path+"\n";
		run("Bio-Formats Importer", "open=["+file_path+"] autoscale color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
		run("Stack to Images");
		chan = nImages();
		numrenameall(chan);	// renaming files to plain numbers (custom function, see function section)	
		selectWindow("2");
		run("Duplicate...", "title=2_org");
		setOption("BlackBackground", true);

	// Erasing the "Wurzelhaube" from the imaging data (all channels) or any other obscuring noise which is above high-threshold (hth)
	// and below low-threshold (lth)
		print("deleting Wurzelhaube");
		selectWindow("6");
		getStatistics(area, mean, min, max, std);
		print("max: "+max+"  min: "+min+"  std:"+std);
		hth = max;
		lth = 100;
		threshallone(hth, lth, "6", chan, 0);     // apply masks to all channels (custom function, see function section) 
		if (deb == true){
			selectWindow("dmask1");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"wurzelhaube_delete_mask.png");
			} 
		close("dmask1");		


	// Remove noise signals (below lth) and saturated pixels (above hth) based on Aqua channel from the imaging data (all channels).  
		print("removing noise and saturation");
		if (deb == true){
			selectWindow("2");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"ch2_BEFORE_AquaMask_delete_mask.png");
			rename("2");
			} 
		hth = 4095;
		lth = 300; 
		threshallone(hth, lth, "2", chan, 1);	// apply masks to all channels (custom function, see function section)  
		if (deb == true){
			selectWindow("2");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"ch2_AFTER_AquaMask_delete_mask.png");
			rename("2");
			} 
		
	// get basic image parameters and with that "resolution" and "unit"
		print("Getting image parameters");
		getVoxelSize(width, height, depth, unit);
		resolution=1/width;
		print(width);

	// ROI generation includin convert pure Aceptor channel (Neon) to 8-bit for ROI generation.
		print("Generating ROIs");
		selectWindow("6");
		run("32-bit");
		run("Multiply...", "value=255");
		run("Divide...", "value="+max);
		run("Enhance Contrast", "saturated=0.35");
		setOption("ScaleConversions", false);
		run("8-bit");

		run("Mean...", "radius=2 stack");
	
		if (ROI_automatic == "manual"){			// Manual threshold 
                	run("adaptiveThr ");
                	close("preview");
			}
		if (ROI_automatic == "automatic"){			// Threshold based on settings, see user settings above
                	run("adaptiveThr ", "using=[Weighted mean] from="+Thr_from+" then="+Thr_then+" output");
			}                
                if (inv == true){
			run("Invert");	//Invert appear necessary on Mac, but wrong on Win (Ole)
			}
                run("Watershed");
                run("Set Scale...", "distance="+resolution+" known=1 pixel=1 unit="+unit);
                run("Analyze Particles...", "size="+S+"-Infinity circularity=0.50-1.00 show=Overlay display exclude clear add");
                run("Clear Results");
		run("Invert");   // White background 

	// Tip and Center find
		print("Finding tip");
                selectWindow("2");
		if (tipauto == "manual"){   
			tip = ManuTip();
			}
		if (tipauto == "automatic"){
			tip = tipfind(tipfind_thresh, deb, folder+save_folder, "tip", "2");		// finds tip (custom function, see function section) 
			}
		if (tipauto == "automatic" || tipauto == "manual"){
			print("Finding Center-of-mass");
			selectWindow("2_org");
			run("Select None");
                	cen = centerfind();			// finds center of mass (custom function, see function section)  
 			File.append(file+"\t"+tip[0]+"\t"+tip[1]+"\t"+cen[0]/width+"\t"+cen[1]/height+"\n ", folder+"/"+save_folder+""+"/pos.txt");     // save center of mass for special analysis in pos.txt
			}
		if (tipauto == "line"){			// Ole set all manual
			print("Set QC");
			tip = ManuTip();
			makePoint(tip[0]/width, tip[1]/height, "large yellow hybrid add");
			wait(250);
			print("Set axis");
			cen = ManuTip();
			print("get corrected");
			makeLine(tip[0]/width, tip[1]/height, cen[0]/width, cen[1]/height);
			Dialog.createNonBlocking("Correct axis if nececssary")
			Dialog.show()
			getLine(x1, y1, x2, y2, lineWidth);
			tip[0] = x1;
			tip[1] = y1;
			cen[0] = x2;
			cen[1] = y2;
			File.append(file+"\t"+tip[0]+"\t"+tip[1]+"\t"+cen[0]+"\t"+cen[1]+"\n ", folder+"/"+save_folder+""+"/pos.txt");     // save center of mass for special analys			
			}

	// FRET calculation
		print("Calculating FRET ratios");
		if (FRET_analysis == "division"){			// "Division fret" create FRET ratio based on simple division between channels 4 and 2    
			imageCalculator("Divide create 32-bit", "4", "2");
			rename("divfret");
        	        createfretmask(FRET_thresh_min, FRET_thresh_max);
	       		imagemultip("divfret", "FRET_mask");				
			if (deb == true){
				As("Png", folder+save_folder+"/"+file+"_"+"divfret_raw.png");
				rename("divfret");
			}	 

			}
		if (FRET_analysis == "unmixing"){			//"Unmixing fret" create FRET ratio based on unmixed Aqua and Neon channels 
			run("Unmix ", "ch_1=1 ch_2=2 ch_3=3 ch_4=4 ch_5=5 open=["+matrix_path+"]");
			imageCalculator("Divide create 32-bit", "Neon","Aqua");
			rename("unfret");
			createfretmask(FRET_thresh_min, FRET_thresh_max);
			imagemultip("unfret", "FRET_mask");				
			if (deb == true){
				saveAs("Png", folder+save_folder+"/"+file+"_"+"unfret_raw.png");
				rename("unfret");
				}	 
			}
	
	// Visualization
		print("Visaulization");
	        if (FRET_analysis == "division"){
			what_to_work_on = "divfret"; 
			}
	        if (FRET_analysis == "unmixing"){ 
			what_to_work_on = "unfret";
			}
                selectWindow("Mask");
                run("Clear Results");
                selectWindow(what_to_work_on);
		run("Multiply...", "value=256");
		run("16-bit");
                run("From ROI Manager");
                run("Set Measurements...", "area mean centroid display invert redirect=None decimal=3");
                roiManager("Measure");
 		if (Visu_style == "dynamic"){
			Visu(offset_div, "N"+what_to_work_on, "", area_thresh, tip[0], tip[1], "NN"+what_to_work_on);
			}
		if (Visu_style == "fixed"){
			range = d2s(lower,0)+"-"+d2s(upper,0);
			Visufix(range, "N"+what_to_work_on, "", tip[0], tip[1], "NN"+what_to_work_on);
			}
		run("Close All");
                run("Clear Results");
                selectWindow("ROI Manager");
                run("Close");
		}
	}


File.saveString("\n ",folder+"/"+save_folder+"/CheckString.txt");
File.append(cs, folder+"/"+save_folder+"/CheckString.txt");






// FUNCTIONS ////////////////////////////////////////////////////////////////////////////////////////////////

// MULTIPLY keep image with other image
function imagemultip(keep, with) { 
	imageCalculator("Multiply create", keep, with);
	rename("temp");
	close(keep);
	selectWindow("temp");
	rename(keep);
	}

// MULTIPLY keep image with other image  32 bit
function imagemultip32(keep, with) { 
	imageCalculator("Multiply create 32-bit", keep, with);
	rename("temp");
	close(keep);
	selectWindow("temp");
	rename(keep);
	}

// RENAMES all channels numerical
function numrenameall(chan) {
	for (i=1; i<=chan; i++) { 
		selectImage(i); 
		rename(d2s(i,0));
	  	}
	}



// THRESHOLD all channels based on 2
function threshall(hth, lth, chA, chB,chan) {
	selectWindow(chA);
	run("Duplicate...", "title=dmask1");
	changeValues(hth,pow(2,32),0);
	changeValues(0,lth,0);
	changeValues(1,hth,1);
	selectWindow(chB);
	run("Duplicate...", "title=dmask2");
	changeValues(hth,pow(2,32),0);
	changeValues(0,lth,0);
	changeValues(1,hth,1);
	imageCalculator("Multiply create", "dmask1","dmask2");
	rename("tmask");
	close("dmask1");
	close("dmask2");
	for (i=1; i<=chan; i++) { 
		selectImage(d2s(i,0)); 
		imageCalculator("Multiply create", "tmask", d2s(i,0));
		close(d2s(i,0));
		rename(d2s(i,0));
		}
	}




// THRESHOLD all channels based on 1
function threshallone(hth, lth, chA,chan,b) {
	print(hth+"_"+lth);
	selectWindow(chA);
	run("Duplicate...", "title=dmask1");
	changeValues(hth,pow(2,32),0);
	changeValues(0,lth,0);
	changeValues(1,hth,1);
	//if (b == 1) {ABORT}   FOR DEBUGGINHG
	for (i=1; i<=chan; i++) {
		selectImage(d2s(i,0)); 
		//if (b == 1) {ABORT}  FOR DEBUGGING
		imageCalculator("Multiply create", "dmask1", d2s(i,0));
		close(d2s(i,0));
		rename(d2s(i,0));
		}
	}



// FRET Mask
function createfretmask(fret_thresh_min, fret_thresh_max) {
	run("Duplicate...", "title=FRET_mask");
	selectWindow("FRET_mask");
	changeValues(1/0, 1/0, 0);
	changeValues(0, fret_thresh_min, 0); 
	changeValues(fret_thresh_min,fret_thresh_max,0.5);
	changeValues(1,100000000000000000000000,0);
	changeValues(0.5,0.5,1);
	}



// TIPFIND
function tipfind(tipfind_thresh, deb, dir, lotitle,chantip) {
	selectWindow(chantip);
	run("Duplicate...", " ");
	//run("Invert");
	width = getWidth; // = j
	height = getHeight; // = i
	i=1;
	sfr=0;
	while(sfr<tipfind_thresh){
	        sfr = 0;
	        for(j=1; j<width; j++){
	                v = getPixel(j, i);
	                sfr = sfr + v;
	                }
	        i++;  
		if (i > height){
			print("Tip find not sucessfull on heigth setting to 0");
			i = 0;
			sfr = tipfind_thresh;
			}
	        }
	j=0;
	sfrl=0;
	i = i-1; // since we have abiove i++ after the reading of the pixel line value
	while(sfrl<(sfr/2)){
	        v = getPixel(j, i);
	        sfrl = sfrl + v;
	        j++;   
		if (j > width){
			print("Tip find not sucessfull on width setting to 0");
			j = 0;
			sfrl = tipfind_thresh;
			}
	        }
	print(i);
	print(j);
	x=j;
	y=i;
	run("Duplicate...", " ");
	changeValues(0,255,0);
	for(i=0; i<10; i++){
	        for(j=0; j<10; j++){
	                setPixel(x+j,y+i,255);
	                }
	        }
	run("Merge Channels...", "c1="+chantip+"-1 c4="+chantip+"-2 create");
	if (deb == true){
		saveAs(dir+"/"+lotitle+"_TIP.png");
		}
	rename("Tip");
	ret = newArray(x,y);
	return ret;
	}	

// CENTER OF MASS
function centerfind() {
	run("Duplicate...", " ");
      //  run("Invert");
	changeValues(BG,10000000,1000);
	changeValues(0,BG,0);
	//run("8-bit");
	//run("Make Binary");
	run("Set Measurements...", "center redirect=None decimal=3");
	run("Measure");
	xcen = getResult("XM", 0);
	ycen = getResult("YM", 0);
	ret = newArray(xcen,ycen);
	close("2-1");
	return ret;
	}


// Calculate tip roi position and value and max and min roi values
function Visu(offset_div, prefix, _folder, area_thresh, x, y, mname) {		
	means_list = newArray;
	nROIs = roiManager("count");
	i=0;
	cnt=0;
	while (i<nROIs){ 
		Xi=getResult("X", i);
	        Yi=getResult("Y", i);
	        setResult("Distance (µm) from the tip", i, sqrt((x-Xi)*(x-Xi)+(y-Yi)*(y-Yi)));
		me = getResult("Area", i);
		if (me > area_thresh){
	 		me = getResult("Mean", i);
			means_list[cnt] = me;
			cnt++;
			}
	        i++;
	        }
	File.makeDirectory(folder+_folder+"/"); 
	saveAs("Results", folder+save_folder+"/"+file+"_"+prefix+"_FRET.csv"); 
	// Range Calculation
	Array.getStatistics(means_list, min, max, mean, std);
	print("max "+max);
	print("min "+min);
	print("mean "+mean);
	print("std "+std);
	low = mean-(std*offset_div);
	if(low<0) {low=0;}
	high = mean+(std*offset_div);
	range = d2s(low,0)+"-"+d2s(high,0);
	print("range "+range);
	//fake coloring
	selectWindow("Mask");
	run("Duplicate...", "title="+mname);
	selectWindow(mname);
	run("Invert");
	run("ROI Color Coder", "measurement=Mean lut=physics width=0 opacity=80 label=[FRET ratio] range="+range+" n.=10 decimal=2 ramp=[512 pixels] font=SansSerif font_size=14 draw");
	selectWindow(mname);
	roiManager("Show All without labels");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_FRET.png");
	selectWindow("Mean Ramp");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_RAMP.png");
	}	




// Calculate tip roi position and value and max and min roi values
function Visufix(range, prefix, _folder, x, y, mname) {		
	File.makeDirectory(folder+_folder+"/"); 
	saveAs("Results", folder+save_folder+"/"+file+"_"+prefix+"_FRET.csv"); 

	print("range "+range);
	//fake coloring
	selectWindow("Mask");
	run("Duplicate...", "title="+mname);
	selectWindow(mname);
	run("Invert");
	run("ROI Color Coder", "measurement=Mean lut=physics width=0 opacity=80 label=[FRET ratio] range="+range+" n.=10 decimal=2 ramp=[512 pixels] font=SansSerif font_size=14 draw");
	selectWindow(mname);
	roiManager("Show All without labels");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_FRET.png");
	selectWindow("Mean Ramp");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_RAMP.png");
	}	


// Manual Tip Choose
function ManuTip(){
      	leftButton=16;
	x2=-1; y2=-1; z2=-1; flags2=-1;
      	logOpened = false;
	done = 0;
      	if (getVersion>="1.37r")
		setOption("DisablePopupMenu", true);
	while (done == 0) {
		getCursorLoc(x, y, z, flags);
		if (x!=x2 || y!=y2 || z!=z2 || flags!=flags2) {
              		s = " ";
              		if (flags&leftButton!=0){ 
				s = s + "<left>";
              			print(x+" "+y+" "+z+" "+flags + "" + s);
              			logOpened = true;
				done = 1;
				}
          	}
        x2=x; y2=y; z2=z; flags2=flags;
      	}
      	if (getVersion>="1.37r")
		setOption("DisablePopupMenu", false);
  	ret = newArray(x*width,y*height);
	return ret;
      	}



