// USER SETTINGS ///////////////////////////////////////////////////////////////////////

// working folder. All .czi files in this folder are processed. A subfolder "" is generated which stores all results
folder = "/Users/andre.stiel/Documents/writing/articles/2016_auxin_big/Martina/testproblem/";

// ROI slection
ROI_automatic = 1;	// 0=manual, 1=automatic, if ROI selection is automatic, this only works in very homogenous data and with the right numbers (Thr_from and Thr_then) determined in manual mode before
Thr_from = 25;		// see above
Thr_then = -1;		// see above
S = 15;			// thresholding of the ROI segmentation, default=15 in micron^2

// tip selection
tipauto = 3;		// tipfinder 0=manual (click position with left mouse on open image), 1= autotmatic (set "tipfind_thresh" for sensitivity)
tipfind_thresh = 10000; // The lower the value the more sensitive the tip detection. Might then also pick up noise as tip.
BG = 100;		// Everything below considered as out of root BG

// FRET analysis
FRET_analysis =1;	// 1 = simnple division, 2 = based on unmixing results
FRET_thresh_min = 0.3;
FRET_thresh_max = 5;

// Visulaization
Visu_style = 2 		// 1  = dynamic thresholds based on "offset_div" below, 2 = fixed threshold based on "upper" and "lower" below. 
offset_div = 1.5;   	// x times the std deviation +/- from mean. E.g. for 1:  lower bound: "mean - std * 1", upper bound : "mean + std * 1"
lower = 0;		// lower limit of the colorrange	
upper = 0.5;		// upper limit of the colorrange	
area_thresh=10;  	// only consider ROIs above this pixel number for analysis

// Script
deb = 0; 		// if deb = 1 a number of intermediate images are saved
inv = 1;		// toggles invert on / off, might be necessary on a number of systems, 1 = default

// path to matrix used for multi-channel unmixing ("unfret"). This path has to point to a prior generated unmixing-matrix .txt. Generated with  !!!!!!!!!!!!!!!!!!! 
matrix_path = "/Users/andre.stiel/Documents/writing/articles/2016_auxin_big/Martina/2019-08-20_Unmixuing_matrix_AuxSen_channal_1-5.txt";  		







// BODY /////////////////////////////////////////////////////////////////////////////////
dir_list = getFileList(folder);
// !!!!!!!!!!!!!! ADD POS FILE GENERATE
getDateAndTime(year, month, dayOfWeek, dayOfMonth, hour, minute, second, msec);
save_folder = d2s(year,0)+"_"+d2s(month,0)+"_"+d2s(dayOfMonth,0)+"_"+d2s(hour,0)+"_"+d2s(minute,0)+"_"+d2s(second,0);
print(save_folder);
File.makeDirectory(folder+"/"+save_folder);
File.saveString("\n ",folder+"/"+save_folder+"/pos.txt");
setOption("ExpandableArrays", true);


for (i=0; i<dir_list.length; i++) {
	if (endsWith(dir_list[i], ".czi")) {
		print(dir_list[i]+"----------------------------------");

	// Load file using "Bio-Formats Importer"
		print("Loading Files");
		file = dir_list[i];
		file_path = folder+file;
		run("Bio-Formats Importer", "open=["+file_path+"] autoscale color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
		run("Stack to Images");
		chan = nImages();
		numrenameall(chan);	// renaming files to plain numbers (custom function, see function section)	
		selectWindow("2");
		run("Duplicate...", "title=2_org");
		setOption("BlackBackground", true);

	// Erasing the "Wurzelhaube" from the imaging data (all channels) or any other obscuring noise which is above high-threshold (hth)
	// and below low-threshold (lth)
		print("deleting Wurzelhaube");
		selectWindow("6");
		getStatistics(area, mean, min, max, std);
		print("max: "+max+"  min: "+min+"  std:"+std);
		hth = max;
		lth = 100;
		threshallone(hth, lth, "6", chan, 0);     // apply masks to all channels (custom function, see function section) 
		if (deb == 1){
			selectWindow("dmask1");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"wurzelhaube_delete_mask.png");
			} 
		close("dmask1");		


	// Remove noise signals (below lth) and saturated pixels (above hth) based on Aqua channel from the imaging data (all channels).  
		print("removing noise and saturation");
		if (deb == 1){
			selectWindow("2");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"ch2_BEFORE_AquaMask_delete_mask.png");
			rename("2");
			} 
		hth = 4095;
		lth = 300; 
		threshallone(hth, lth, "2", chan, 1);	// apply masks to all channels (custom function, see function section)  
		if (deb == 1){
			selectWindow("2");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"ch2_AFTER_AquaMask_delete_mask.png");
			rename("2");
			} 
		
	// get basic image parameters and with that "resolution" and "unit"
		print("Getting image parameters");
		getVoxelSize(width, height, depth, unit);
		resolution=1/width;
		print(width);


ERlow = 0.9;
ERhigh = 1.1;



	// FRET calculation
		print("Calculating FRET ratios");
		if (FRET_analysis == 1){			// "Division fret" create FRET ratio based on simple division between channels 4 and 2    
			imageCalculator("Divide create 32-bit", "4", "2");
			rename("divfret");
        	        createfretmask(FRET_thresh_min, FRET_thresh_max);
	       		imagemultip("divfret", "FRET_mask");
			run("physics");				
			setMinAndMax(lower, upper);
			run("Calibration Bar...", "location=[Upper Right] fill=White label=Black number=5 decimal=0 font=12 zoom=1 overlay");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"divfret_raw.png");
			rename("divfret");

			changeValues(0,ERlow,NaN);
			changeValues(ERhigh,100000000000000,NaN);
			getStatistics(area, mean, min, max, std);
			print(mean);
			File.append(d2s(mean,3)+";"+"\n",folder+"/"+save_folder+"/pos.txt");

			}

		if (FRET_analysis == 2){			//"Unmixing fret" create FRET ratio based on unmixed Aqua and Neon channels 
			run("Unmix ", "ch_1=1 ch_2=2 ch_3=3 ch_4=4 ch_5=5 open=["+matrix_path+"]");
			imageCalculator("Divide create 32-bit", "Neon","Aqua");
			rename("unfret");
			createfretmask(FRET_thresh_min, FRET_thresh_max);
			imagemultip("unfret", "FRET_mask");	
			run("physics");			
			setMinAndMax(lower, upper);
			run("Calibration Bar...", "location=[Upper Right] fill=White label=Black number=5 decimal=0 font=12 zoom=1 overlay");
			saveAs("Png", folder+save_folder+"/"+file+"_"+"unfret_raw.png");
			rename("unfret");
			 changeValues(0,ERlow,NaN);
                         changeValues(ERhigh,100000000000000,NaN);
                         getStatistics(area, mean, min, max, std);
                         print(mean);
			File.append(d2s(mean,3)+";"+"\n",folder+"/"+save_folder+"/pos.txt");
			}
		
		run("Close All");
		}
	}









// FUNCTIONS ////////////////////////////////////////////////////////////////////////////////////////////////

// MULTIPLY keep image with other image
function imagemultip(keep, with) { 
	imageCalculator("Multiply create", keep, with);
	rename("temp");
	close(keep);
	selectWindow("temp");
	rename(keep);
	}

// MULTIPLY keep image with other image  32 bit
function imagemultip32(keep, with) { 
	imageCalculator("Multiply create 32-bit", keep, with);
	rename("temp");
	close(keep);
	selectWindow("temp");
	rename(keep);
	}

// RENAMES all channels numerical
function numrenameall(chan) {
	for (i=1; i<=chan; i++) { 
		selectImage(i); 
		rename(d2s(i,0));
	  	}
	}



// THRESHOLD all channels based on 2
function threshall(hth, lth, chA, chB,chan) {
	selectWindow(chA);
	run("Duplicate...", "title=dmask1");
	changeValues(hth,pow(2,32),0);
	changeValues(0,lth,0);
	changeValues(1,hth,1);
	selectWindow(chB);
	run("Duplicate...", "title=dmask2");
	changeValues(hth,pow(2,32),0);
	changeValues(0,lth,0);
	changeValues(1,hth,1);
	imageCalculator("Multiply create", "dmask1","dmask2");
	rename("tmask");
	close("dmask1");
	close("dmask2");
	for (i=1; i<=chan; i++) { 
		selectImage(d2s(i,0)); 
		imageCalculator("Multiply create", "tmask", d2s(i,0));
		close(d2s(i,0));
		rename(d2s(i,0));
		}
	}




// THRESHOLD all channels based on 1
function threshallone(hth, lth, chA,chan,b) {
	print(hth+"_"+lth);
	selectWindow(chA);
	run("Duplicate...", "title=dmask1");
	changeValues(hth,pow(2,32),0);
	changeValues(0,lth,0);
	changeValues(1,hth,1);
	//if (b == 1) {ABORT}   FOR DEBUGGINHG
	for (i=1; i<=chan; i++) {
		selectImage(d2s(i,0)); 
		//if (b == 1) {ABORT}  FOR DEBUGGING
		imageCalculator("Multiply create", "dmask1", d2s(i,0));
		close(d2s(i,0));
		rename(d2s(i,0));
		}
	}



// FRET Mask
function createfretmask(fret_thresh_min, fret_thresh_max) {
	run("Duplicate...", "title=FRET_mask");
	selectWindow("FRET_mask");
	changeValues(1/0, 1/0, 0);
	changeValues(0, fret_thresh_min, 0); 
	changeValues(fret_thresh_min,fret_thresh_max,0.5);
	changeValues(1,100000000000000000000000,0);
	changeValues(0.5,0.5,1);
	}



// TIPFIND
function tipfind(tipfind_thresh, deb, dir, lotitle,chantip) {
	selectWindow(chantip);
	run("Duplicate...", " ");
	//run("Invert");
	width = getWidth; // = j
	height = getHeight; // = i
	i=1;
	sfr=0;
	while(sfr<tipfind_thresh){
	        sfr = 0;
	        for(j=1; j<width; j++){
	                v = getPixel(j, i);
	                sfr = sfr + v;
	                }
	        i++;  
		if (i > height){
			print("Tip find not sucessfull on heigth setting to 0");
			i = 0;
			sfr = tipfind_thresh;
			}
	        }
	j=0;
	sfrl=0;
	i = i-1; // since we have abiove i++ after the reading of the pixel line value
	while(sfrl<(sfr/2)){
	        v = getPixel(j, i);
	        sfrl = sfrl + v;
	        j++;   
		if (j > width){
			print("Tip find not sucessfull on width setting to 0");
			j = 0;
			sfrl = tipfind_thresh;
			}
	        }
	print(i);
	print(j);
	x=j;
	y=i;
	run("Duplicate...", " ");
	changeValues(0,255,0);
	for(i=0; i<10; i++){
	        for(j=0; j<10; j++){
	                setPixel(x+j,y+i,255);
	                }
	        }
	run("Merge Channels...", "c1="+chantip+"-1 c4="+chantip+"-2 create");
	if (deb == 1){
		saveAs(dir+"/"+lotitle+"_TIP.png");
		}
	rename("Tip");
	ret = newArray(x,y);
	return ret;
	}	

// CENTER OF MASS
function centerfind() {
	run("Duplicate...", " ");
      //  run("Invert");
	changeValues(BG,10000000,1000);
	changeValues(0,BG,0);
	//run("8-bit");
	//run("Make Binary");
	run("Set Measurements...", "center redirect=None decimal=3");
	run("Measure");
	xcen = getResult("XM", 0);
	ycen = getResult("YM", 0);
	ret = newArray(xcen,ycen);
	close("2-1");
	return ret;
	}


// Calculate tip roi position and value and max and min roi values
function Visu(offset_div, prefix, _folder, area_thresh, x, y, mname) {		
	means_list = newArray;
	nROIs = roiManager("count");
	i=0;
	cnt=0;
	while (i<nROIs){ 
		Xi=getResult("X", i);
	        Yi=getResult("Y", i);
	        setResult("Distance (µm) from the tip", i, sqrt((x-Xi)*(x-Xi)+(y-Yi)*(y-Yi)));
		me = getResult("Area", i);
		if (me > area_thresh){
	 		me = getResult("Mean", i);
			means_list[cnt] = me;
			cnt++;
			}
	        i++;
	        }
	File.makeDirectory(folder+_folder+"/"); 
	saveAs("Results", folder+save_folder+"/"+file+"_"+prefix+"_FRET.csv"); 
	// Range Calculation
	Array.getStatistics(means_list, min, max, mean, std);
	print("max "+max);
	print("min "+min);
	print("mean "+mean);
	print("std "+std);
	low = mean-(std*offset_div);
	if(low<0) {low=0;}
	high = mean+(std*offset_div);
	range = d2s(low,0)+"-"+d2s(high,0);
	print("range "+range);
	//fake coloring
	selectWindow("Mask");
	run("Duplicate...", "title="+mname);
	selectWindow(mname);
	run("Invert");
	run("ROI Color Coder", "measurement=Mean lut=physics width=0 opacity=80 label=[FRET ratio] range="+range+" n.=10 decimal=2 ramp=[512 pixels] font=SansSerif font_size=14 draw");
	selectWindow(mname);
	roiManager("Show All without labels");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_FRET.png");
	selectWindow("Mean Ramp");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_RAMP.png");
	}	




// Calculate tip roi position and value and max and min roi values
function Visufix(range, prefix, _folder, x, y, mname) {		
	File.makeDirectory(folder+_folder+"/"); 
	saveAs("Results", folder+save_folder+"/"+file+"_"+prefix+"_FRET.csv"); 

	print("range "+range);
	//fake coloring
	selectWindow("Mask");
	run("Duplicate...", "title="+mname);
	selectWindow(mname);
	run("Invert");
	run("ROI Color Coder", "measurement=Mean lut=physics width=0 opacity=80 label=[FRET ratio] range="+range+" n.=10 decimal=2 ramp=[512 pixels] font=SansSerif font_size=14 draw");
	selectWindow(mname);
	roiManager("Show All without labels");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_FRET.png");
	selectWindow("Mean Ramp");
	saveAs("Png", folder+save_folder+"/"+file+"_"+prefix+"_RAMP.png");
	}	


// Manual Tip Choose
function ManuTip(){
      	leftButton=16;
	x2=-1; y2=-1; z2=-1; flags2=-1;
      	logOpened = false;
	done = 0;
      	if (getVersion>="1.37r")
		setOption("DisablePopupMenu", true);
	while (done == 0) {
		getCursorLoc(x, y, z, flags);
		if (x!=x2 || y!=y2 || z!=z2 || flags!=flags2) {
              		s = " ";
              		if (flags&leftButton!=0){ 
				s = s + "<left>";
              			print(x+" "+y+" "+z+" "+flags + "" + s);
              			logOpened = true;
				done = 1;
				}
          	}
        x2=x; y2=y; z2=z; flags2=flags;
      	}
      	if (getVersion>="1.37r")
		setOption("DisablePopupMenu", false);
  	ret = newArray(x*width,y*height);
	return ret;
      	}



